/*
 * DO NOT EDIT THIS FILE - it is generated by Glade.
 */

GtkWidget* create_main_window (void);
GtkWidget* create_playlist_window (void);
GtkWidget* create_playlist_load (void);
GtkWidget* create_playlist_save (void);
GtkWidget* create_effects_window (void);
GtkWidget* create_scopes_window (void);
GtkWidget* create_fileselection1 (void);
