#include "config.h" // For VERSION macro.

static const char * const licenseText =
"<h1>AlsaPlayer "VERSION"</h1>"
"<p>Copyright (C) 1998-2001 Andy Lo A Foe &lt;andy@alsa-project.org&gt;</p>"
"<p>"
"This program is free software; you can redistribute it and/or modify"
" it under the terms of the GNU General Public License as published by"
" the Free Software Foundation; either version 2 of the License, or"
" (at your option) any later version."
"</p>"
"<p>"
"This program is distributed in the hope that it will be useful,"
" but WITHOUT ANY WARRANTY; without even the implied warranty of"
" MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the"
" GNU General Public License for more details."
"</p>"
"<p>"
"You should have received a copy of the GNU General Public License"
" along with this program; if not, write to the Free Software"
" Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA."
"</p>";

